local PREVIEW_TEXTURE = Resources.load_texture("preview.png")
local BUTTON_TEXTURE = Resources.load_texture("button.png")

---@param augment Augment
function augment_init(augment)
  local player = augment:owner()

  local button = augment:create_card_button(2)
  button:set_texture(BUTTON_TEXTURE)
  button:set_preview_texture(PREVIEW_TEXTURE)

  local animation = button:animation()
  animation:load("button.animation")

  -- use USED state just to get the ICON point
  animation:set_state("USED")
  local icon_sprite = button:sprite():create_node()
  local icon_point = animation:get_point("ICON")
  icon_sprite:set_offset(icon_point.x, icon_point.y)

  -- setup state
  local used = false
  local armed_card = nil

  augment.charged_attack_func = function()
    if not armed_card then
      return nil
    end

    return Action.from_card(player, armed_card)
  end

  function reset()
    animation:set_state("DISABLED")
    icon_sprite:hide()
    armed_card = nil
    used = false
  end

  local last_card = nil
  local open_component = player:create_component(Lifetime.CardSelectOpen)
  open_component.on_update_func = function()
    if last_card == armed_card then
      armed_card = nil
    end
    last_card = armed_card
    reset()
  end

  reset()

  button.on_selection_change_func = function()
    if armed_card then
      return
    end

    local staged_items = player:staged_items()
    local latest_staged_item = staged_items[#staged_items]

    if not latest_staged_item or latest_staged_item.category ~= "deck_card" then
      animation:set_state("DISABLED")
      return
    end

    local deck_card_index = latest_staged_item.index
    local deck_card = player:deck_card(deck_card_index)
    local card = CardProperties.from_package(deck_card.package_id, deck_card.code)

    if card.time_freeze or card.element ~= Element.None or card.card_class ~= CardClass.Standard or card.recover > 0 then
      animation:set_state("DISABLED")
      return
    end

    animation:set_state("DEFAULT")
  end

  button.use_func = function()
    if used or animation:state() ~= "DEFAULT" then
      return false
    end

    used = true

    local staged_items = player:staged_items()
    local deck_card_index = staged_items[#staged_items].index

    -- setup armed_card
    local deck_card = player:deck_card(deck_card_index)
    armed_card = CardProperties.from_package(deck_card.package_id, deck_card.code)

    -- use the armed_card's icon texture
    local icon_texture = player:staged_item_texture(#staged_items)
    icon_sprite:set_texture(icon_texture)
    icon_sprite:hide()

    -- animate
    local TOTAL_FRAMES = 30
    local INTERVAL = 2

    local ticks = 0
    local blinked = false

    local component = player:create_component(Lifetime.Scene)

    player:set_card_selection_blocked(true)

    component.on_update_func = function()
      ticks = ticks + 1

      local past_half = ticks >= TOTAL_FRAMES / 2

      if past_half and ticks % INTERVAL ~= 0 then
        -- idle
        return
      end

      -- animate
      blinked = not blinked

      if blinked then
        player:stage_deck_card(deck_card_index)
        animation:set_state("DEFAULT")
        icon_sprite:hide()
      else
        player:stage_deck_discard(deck_card_index)
        animation:set_state("USED")

        if past_half then
          icon_sprite:reveal()
        end
      end

      -- complete
      if ticks < TOTAL_FRAMES then
        return
      end

      -- set undo func
      player:stage_deck_discard(deck_card_index, function()
        player:stage_deck_card(deck_card_index)
        reset()
      end)

      -- clean up
      player:set_card_selection_blocked(false)
      animation:set_state("USED")
      component:eject()
    end

    return true
  end
end
